@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silclog.c,v 1.27 2002/06/19 19:41:52 priikone Exp $ */
+/* $Id: silclog.c,v 1.29 2002/09/15 09:40:48 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -49,6 +49,9 @@ static struct SilcLogStruct silclogs[SILC_LOG_MAX] = {
   {"", NULL, 0, "Fatal", SILC_LOG_FATAL, NULL, NULL},
 };
 
+/* Causes logging output to contain timestamps */
+bool silc_log_timestamp = TRUE;
+
 /* If TRUE, log files will be flushed for each log input */
 bool silc_log_quick = FALSE;
 
@@ -119,7 +122,7 @@ static void silc_log_checksize(SilcLog log)
   /* It's too big */
   fprintf(log->fp, "[%s] [%s] Cycling log file, over max "
 	  "logsize (%lu kilobytes)\n",
-	  silc_get_time(), log->typename, log->maxsize / 1024);
+	  silc_get_time(0), log->typename, log->maxsize / 1024);
   fflush(log->fp);
   fclose(log->fp);
   snprintf(newname, sizeof(newname), "%s.old", log->filename);
@@ -220,7 +223,12 @@ void silc_log_output(SilcLogType type, char *string)
   goto end;
 
  found:
-  fprintf(fp, "[%s] [%s] %s\n", silc_get_time(), typename, string);
+  /* writes the logging string to the selected channel */
+  if (silc_log_timestamp)
+    fprintf(fp, "[%s] [%s] %s\n", silc_get_time(0), typename, string);
+  else
+    fprintf(fp, "[%s] %s\n", typename, string);
+
   if (silc_log_quick || silc_log_starting) {
     fflush(fp);
     if (log) /* we may have been redirected to stderr */
