@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcschedule.c,v 1.36 2002/04/11 19:42:03 priikone Exp $ */
+/* $Id: silcschedule.c,v 1.39 2002/09/16 10:20:21 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcschedule_i.h"
@@ -35,7 +35,7 @@ int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
    the wakeup mechanism of the scheduler.  In multi-threaded environment
    the scheduler needs to be wakenup when tasks are added or removed from
    the task queues.  Returns context to the platform specific scheduler. */
-void *silc_schedule_internal_init(SilcSchedule schedule);
+void *silc_schedule_internal_init(SilcSchedule schedule, void *context);
 
 /* Uninitializes the platform specific scheduler context. */
 void silc_schedule_internal_uninit(void *context);
@@ -70,6 +70,8 @@ void silc_schedule_internal_signals_unblock(void *context);
 
 /* Internal task management routines. */
 
+static void silc_schedule_dispatch_timeout(SilcSchedule schedule,
+					   bool dispatch_all);
 static void silc_task_queue_alloc(SilcTaskQueue *queue);
 static void silc_task_queue_free(SilcTaskQueue queue);
 static SilcTask silc_task_find(SilcTaskQueue queue, SilcUInt32 fd);
@@ -208,6 +210,7 @@ struct SilcTaskQueueStruct {
 
 */
 struct SilcScheduleStruct {
+  void *app_context;		/* Application specific context */
   SilcTaskQueue fd_queue;
   SilcTaskQueue timeout_queue;
   SilcTaskQueue generic_queue;
@@ -225,9 +228,10 @@ struct SilcScheduleStruct {
 /* Initializes the scheduler. This returns the scheduler context that
    is given as arugment usually to all silc_schedule_* functions.
    The `max_tasks' indicates the number of maximum tasks that the
-   scheduler can handle. */
+   scheduler can handle. The `app_context' is application specific
+   context that is delivered to task callbacks. */
 
-SilcSchedule silc_schedule_init(int max_tasks)
+SilcSchedule silc_schedule_init(int max_tasks, void *app_context)
 {
   SilcSchedule schedule;
 
@@ -249,12 +253,13 @@ SilcSchedule silc_schedule_init(int max_tasks)
   schedule->max_fd = max_tasks;
   schedule->timeout = NULL;
   schedule->valid = TRUE;
+  schedule->app_context = app_context;
 
   /* Allocate scheduler lock */
   silc_mutex_alloc(&schedule->lock);
 
   /* Initialize the platform specific scheduler. */
-  schedule->internal = silc_schedule_internal_init(schedule);
+  schedule->internal = silc_schedule_internal_init(schedule, app_context);
 
   return schedule;
 }
@@ -271,6 +276,19 @@ bool silc_schedule_uninit(SilcSchedule schedule)
   if (schedule->valid == TRUE)
     return FALSE;
 
+  /* Dispatch all timeouts before going away */
+  silc_mutex_lock(schedule->timeout_queue->lock);
+  silc_schedule_dispatch_timeout(schedule, TRUE);
+  silc_mutex_unlock(schedule->timeout_queue->lock);
+
+  /* Deliver signals before going away */
+  if (schedule->signal_tasks) {
+    SILC_SCHEDULE_UNLOCK(schedule);
+    silc_schedule_internal_signals_call(schedule->internal, schedule);
+    schedule->signal_tasks = FALSE;
+    SILC_SCHEDULE_LOCK(schedule);
+  }
+
   /* Unregister all tasks */
   silc_schedule_task_remove(schedule->fd_queue, SILC_ALL_TASKS);
   silc_schedule_task_remove(schedule->timeout_queue, SILC_ALL_TASKS);
@@ -287,6 +305,7 @@ bool silc_schedule_uninit(SilcSchedule schedule)
   silc_schedule_internal_uninit(schedule->internal);
 
   silc_mutex_free(schedule->lock);
+  silc_free(schedule);
 
   return TRUE;
 }
@@ -350,7 +369,8 @@ static void silc_schedule_dispatch_nontimeout(SilcSchedule schedule)
       if (task->valid && schedule->fd_list[i].revents & SILC_TASK_READ) {
 	silc_mutex_unlock(schedule->fd_queue->lock);
 	SILC_SCHEDULE_UNLOCK(schedule);
-	task->callback(schedule, SILC_TASK_READ, task->fd, task->context);
+	task->callback(schedule, schedule->app_context,
+		       SILC_TASK_READ, task->fd, task->context);
 	SILC_SCHEDULE_LOCK(schedule);
 	silc_mutex_lock(schedule->fd_queue->lock);
       }
@@ -359,7 +379,8 @@ static void silc_schedule_dispatch_nontimeout(SilcSchedule schedule)
       if (task->valid && schedule->fd_list[i].revents & SILC_TASK_WRITE) {
 	silc_mutex_unlock(schedule->fd_queue->lock);
 	SILC_SCHEDULE_UNLOCK(schedule);
-	task->callback(schedule, SILC_TASK_WRITE, task->fd, task->context);
+	task->callback(schedule, schedule->app_context,
+		       SILC_TASK_WRITE, task->fd, task->context);
 	SILC_SCHEDULE_LOCK(schedule);
 	silc_mutex_lock(schedule->fd_queue->lock);
       }
@@ -389,7 +410,8 @@ static void silc_schedule_dispatch_nontimeout(SilcSchedule schedule)
 	if (task->valid && schedule->fd_list[i].revents & SILC_TASK_READ) {
 	  silc_mutex_unlock(schedule->generic_queue->lock);
 	  SILC_SCHEDULE_UNLOCK(schedule);
-	  task->callback(schedule, SILC_TASK_READ, fd, task->context);
+	  task->callback(schedule, schedule->app_context,
+			 SILC_TASK_READ, fd, task->context);
 	  SILC_SCHEDULE_LOCK(schedule);
 	  silc_mutex_lock(schedule->generic_queue->lock);
 	}
@@ -398,7 +420,8 @@ static void silc_schedule_dispatch_nontimeout(SilcSchedule schedule)
 	if (task->valid && schedule->fd_list[i].revents & SILC_TASK_WRITE) {
 	  silc_mutex_unlock(schedule->generic_queue->lock);
 	  SILC_SCHEDULE_UNLOCK(schedule);
-	  task->callback(schedule, SILC_TASK_WRITE, fd, task->context);
+	  task->callback(schedule, schedule->app_context,
+			 SILC_TASK_WRITE, fd, task->context);
 	  SILC_SCHEDULE_LOCK(schedule);
 	  silc_mutex_lock(schedule->generic_queue->lock);
 	}
@@ -436,7 +459,8 @@ static void silc_schedule_dispatch_nontimeout(SilcSchedule schedule)
    phase. */
 /* This holds the schedule->lock and the schedule->timeout_queue->lock */
 
-static void silc_schedule_dispatch_timeout(SilcSchedule schedule)
+static void silc_schedule_dispatch_timeout(SilcSchedule schedule,
+					   bool dispatch_all)
 {
   SilcTaskQueue queue = schedule->timeout_queue;
   SilcTask task;
@@ -454,11 +478,13 @@ static void silc_schedule_dispatch_timeout(SilcSchedule schedule)
        the expired tasks. */
     while(1) {
       /* Execute the task if the timeout has expired */
-      if (silc_schedule_task_timeout_compare(&task->timeout, &curtime)) {
+      if (dispatch_all ||
+	  silc_schedule_task_timeout_compare(&task->timeout, &curtime)) {
         if (task->valid) {
 	  silc_mutex_unlock(queue->lock);
 	  SILC_SCHEDULE_UNLOCK(schedule);
-	  task->callback(schedule, SILC_TASK_EXPIRE, task->fd, task->context);
+	  task->callback(schedule, schedule->app_context,
+			 SILC_TASK_EXPIRE, task->fd, task->context);
 	  SILC_SCHEDULE_LOCK(schedule);
 	  silc_mutex_lock(queue->lock);
 	}
@@ -508,8 +534,8 @@ static void silc_schedule_select_timeout(SilcSchedule schedule)
       /* If the timeout is in past, we will run the task and all other
 	 timeout tasks from the past. */
       if (silc_schedule_task_timeout_compare(&task->timeout, &curtime)) {
-	silc_schedule_dispatch_timeout(schedule);
-						
+	silc_schedule_dispatch_timeout(schedule, FALSE);
+
 	/* The task(s) has expired and doesn't exist on the task queue
 	   anymore. We continue with new timeout. */
 	queue = schedule->timeout_queue;
@@ -614,7 +640,7 @@ bool silc_schedule_one(SilcSchedule schedule, int timeout_usecs)
   case 0:
     /* Timeout */
     silc_mutex_lock(schedule->timeout_queue->lock);
-    silc_schedule_dispatch_timeout(schedule);
+    silc_schedule_dispatch_timeout(schedule, FALSE);
     silc_mutex_unlock(schedule->timeout_queue->lock);
     break;
   default:
@@ -670,6 +696,16 @@ void silc_schedule_wakeup(SilcSchedule schedule)
 #endif
 }
 
+/* Returns the application specific context that was saved into the
+   scheduler in silc_schedule_init function.  The context is also
+   returned to application in task callback functions, but this function
+   may be used to get it as well if needed. */
+
+void *silc_schedule_get_context(SilcSchedule schedule)
+{
+  return schedule->app_context;
+}
+
 /* Add new task to the scheduler */
 
 SilcTask silc_schedule_task_add(SilcSchedule schedule, SilcUInt32 fd,
@@ -682,6 +718,9 @@ SilcTask silc_schedule_task_add(SilcSchedule schedule, SilcUInt32 fd,
   SilcTaskQueue queue;
   int timeout = FALSE;
 
+  if (!schedule->valid)
+    return NULL;
+
   SILC_LOG_DEBUG(("Registering new task, fd=%d type=%d priority=%d", fd, 
 		  type, priority));
 
@@ -849,6 +888,9 @@ void silc_schedule_set_listen_fd(SilcSchedule schedule,
   int i;
   bool found = FALSE;
 
+  if (!schedule->valid)
+    return;
+
   SILC_SCHEDULE_LOCK(schedule);
 
   for (i = 0; i < schedule->max_fd; i++)
