@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.75 2002/06/28 13:27:58 priikone Exp $ */
+/* $Id: idlist.c,v 1.76 2002/09/16 10:21:28 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "idlist.h"
@@ -90,13 +90,13 @@ void silc_idlist_del_data(void *entry)
 
 SILC_TASK_CALLBACK_GLOBAL(silc_idlist_purge)
 {
+  SilcServer server = app_context;
   SilcIDListPurge i = (SilcIDListPurge)context;
 
   SILC_LOG_DEBUG(("Purging cache"));
 
   silc_idcache_purge(i->cache);
-  silc_schedule_task_add(i->schedule, 0, 
-			 silc_idlist_purge,
+  silc_schedule_task_add(server->schedule, 0, silc_idlist_purge,
 			 (void *)i, i->timeout, 0,
 			 SILC_TASK_TIMEOUT, SILC_TASK_PRI_LOW);
 }
