@@ -19,7 +19,7 @@
 /*
  * Server packet routines to send packets. 
  */
-/* $Id: packet_send.c,v 1.112 2002/08/30 13:34:12 priikone Exp $ */
+/* $Id: packet_send.c,v 1.115 2002/10/05 18:46:19 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -980,8 +980,8 @@ void silc_server_packet_send_local_channel(SilcServer server,
   /* Send the message to clients on the channel's client list. */
   silc_hash_table_list(channel->user_list, &htl);
   while (silc_hash_table_get(&htl, NULL, (void **)&chl)) {
-    if (chl->client && !chl->client->router) {
-      sock = (SilcSocketConnection)chl->client->connection;
+    if (chl->client && SILC_IS_LOCAL(chl->client)) {
+      sock = chl->client->connection;
 
       /* Send the packet to the client */
       silc_server_packet_send_dest(server, sock, type, flags, chl->client->id,
@@ -1770,6 +1770,9 @@ void silc_server_send_channel_key(SilcServer server,
   chid = silc_id_id2str(channel->id, SILC_ID_CHANNEL);
   if (!chid)
     return;
+
+  if (!channel->channel_key)
+    return;
  
   /* Encode channel key packet */
   tmp_len = strlen(channel->channel_key->cipher->name);
@@ -1802,7 +1805,7 @@ void silc_server_send_command(SilcServer server,
 
   packet = silc_command_payload_encode_vap(command, ident, argc, ap);
   silc_server_packet_send(server, sock, SILC_PACKET_COMMAND, 0,
-			  packet->data, packet->len, TRUE);
+			  packet->data, packet->len, FALSE);
   silc_buffer_free(packet);
   va_end(ap);
 }
@@ -1853,7 +1856,7 @@ void silc_server_send_dest_command_reply(SilcServer server,
 						 ident, argc, ap);
   silc_server_packet_send_dest(server, sock, SILC_PACKET_COMMAND_REPLY, 0,
 			       dst_id, dst_id_type, packet->data, 
-			       packet->len, TRUE);
+			       packet->len, FALSE);
   silc_buffer_free(packet);
   va_end(ap);
 }
